/**
	\file
	\brief Version information - variables and functions to check C-Munipack's version

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_version.h,v 1.12 2016/07/06 19:08:24 dmotl Exp $
*/

#ifndef _CMPACK_VERSION_H_INCLUDED
#define _CMPACK_VERSION_H_INCLUDED

#include "cmpack_config.h"

#ifdef __cplusplus
extern "C" {
#endif
	
/**
	\brief Get major version number
	\details The variable is stored in a header file, so it 
	represents the version available at compile-time of the
	client software. See cmpack_major_version().
*/
#define CMPACK_MAJOR_VERSION 2

/**
	\brief Get minor version number
	\details The variable is stored in a header file, so it 
	represents the version available at compile-time of the
	client software. See cmpack_minor_version().
*/
#define CMPACK_MINOR_VERSION 1

/**
	\brief Get revision number
	\details The variable is stored in a header file, so it 
	represents the version available at compile-time of the
	client software. See cmpack_minor_version().
*/
#define CMPACK_REVISION_NUMBER 37

/**
	\brief Get major version number
	\details The variable is linked in the library, so it represents the version
	which the client software was linked against.
	
	This part is incremented on major changes in the library.

	\return major version number
*/
CMPACK_EXPORT(int, cmpack_major_version, (void));

/**
	\brief Get minor version number
	\details The variable is linked in the library, so it represents the version
	which the client software was linked against.
	
	This part is incremented on changes in the library's
	interface. Even minor numbers indicates stable versions, odd minor numbers
	are reserved for development versions.

	\return minor version number
*/
CMPACK_EXPORT(int, cmpack_minor_version, (void));

/**
	\brief Get revision number
	\details The variable is linked in the library, so it represents the version
	which the client software was linked against.
	
	This part is incremented every time the package(s) are published.

	\return revision number
*/
CMPACK_EXPORT(int, cmpack_revision_number, (void));

/**
	\brief Get C-Munipack library version
	\details The variable is linked in the library, so it represents the version
	which the client software was linked against.
	
	The version identifier consists of three part. The first one
	is major number which is incremented on major changes in the library.
	The second one is minor version incremented on changes in the library's
	interface. Even minor numbers indicates stable versions, odd minor numbers
	are reserved for development versions. The last part, the revision number,
	is incremented every time the package(s) are published.

	\warning The function returns pointer to internal memory buffer. Do not call
	cmpack_free() on the result!

	\return pointer to allocated string or zero on failure.
*/
CMPACK_EXPORT(const char *, cmpack_versionid, (void));

/**
	\brief Check C-Munipack library version
	\details Check the version of installed C-Munipack library. You can pass 
	the constants CMPACK_MAJOR_VERSION, CMPACK_MINOR_VERSION and CMPACK_REVISION_NUMBER 
	as three arguments to the function; the function returns nonzero if the installed
	library is compatible with the version the client application was compiled against.

	\return nonzero if the library is compatible with given version, zero otherwise
*/
CMPACK_EXPORT(int, cmpack_check_version, (int major, int minor, int revision));

#ifdef __cplusplus
}
#endif

#endif
